@@ -1,7 +1,30 @@
+/****************************************************************************
+**
+** Copyright (C) 2015 Oleg Shparber
+** Copyright (C) 2013-2014 Jerzy Kozera
+** Contact: http://zealdocs.org/contact.html
+**
+** This file is part of Zeal.
+**
+** Zeal is free software: you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation, either version 3 of the License, or
+** (at your option) any later version.
+**
+** Zeal is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+** GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with Zeal. If not, see <http://www.gnu.org/licenses/>.
+**
+****************************************************************************/
+
 #include "docset.h"
 
-#include "docsetinfo.h"
 #include "searchquery.h"
+#include "util/plist.h"
 
 #include <QDir>
 #include <QFile>
@@ -14,6 +37,23 @@
 
 using namespace Zeal;
 
+namespace {
+const char IndexNamePrefix[] = "__zi_name"; // zi - Zeal index
+const char IndexNameVersion[] = "0001"; // Current index version
+
+namespace InfoPlist {
+const char CFBundleName[] = "CFBundleName";
+const char CFBundleIdentifier[] = "CFBundleIdentifier";
+const char DashDocSetFamily[] = "DashDocSetFamily";
+const char DashDocSetKeyword[] = "DashDocSetKeyword";
+const char DashDocSetPluginKeyword[] = "DashDocSetPluginKeyword";
+const char DashIndexFilePath[] = "dashIndexFilePath";
+const char DocSetPlatformFamily[] = "DocSetPlatformFamily";
+const char IsDashDocset[] = "isDashDocset";
+const char IsJavaScriptEnabled[] = "isJavaScriptEnabled";
+}
+}
+
 Docset::Docset(const QString &path) :
     m_path(path)
 {
@@ -34,18 +74,24 @@ Docset::Docset(const QString &path) :
     if (!dir.cd(QStringLiteral("Contents")))
         return;
 
-    DocsetInfo info;
+    /// TODO: 'info.plist' is invalid according to Apple, and must alsways be 'Info.plist'
+    /// https://developer.apple.com/library/mac/documentation/MacOSX/Conceptual/BPRuntimeConfig/
+    /// Articles/ConfigFiles.html
+    Util::Plist plist;
     if (dir.exists(QStringLiteral("Info.plist")))
-        info = DocsetInfo::fromPlist(dir.absoluteFilePath(QStringLiteral("Info.plist")));
+        plist.read(dir.absoluteFilePath(QStringLiteral("Info.plist")));
     else if (dir.exists(QStringLiteral("info.plist")))
-        info = DocsetInfo::fromPlist(dir.absoluteFilePath(QStringLiteral("info.plist")));
+        plist.read(dir.absoluteFilePath(QStringLiteral("info.plist")));
     else
         return;
 
+    if (plist.hasError())
+        return;
+
     if (m_name.isEmpty()) {
         // Fallback if meta.json is absent
-        if (!info.bundleName.isEmpty()) {
-            m_name = m_title = info.bundleName;
+        if (!plist.contains(InfoPlist::CFBundleName)) {
+            m_name = m_title = plist[InfoPlist::CFBundleName].toString();
             /// TODO: Remove when MainWindow::docsetName() will not use directory name
             m_name.replace(QLatin1Char(' '), QLatin1Char('_'));
         } else {
@@ -59,8 +105,10 @@ Docset::Docset(const QString &path) :
     }
 
     /// TODO: Verify if this is needed
-    if (info.family == QLatin1String("cheatsheet"))
+    if (plist.contains(InfoPlist::DashDocSetFamily)
+            && plist[InfoPlist::DashDocSetFamily].toString() == QLatin1String("cheatsheet")) {
         m_name = m_name + QLatin1String("cheats");
+    }
 
     if (!dir.cd(QStringLiteral("Resources")) || !dir.exists(QStringLiteral("docSet.dsidx")))
         return;
@@ -75,15 +123,35 @@ Docset::Docset(const QString &path) :
 
     m_type = db.tables().contains(QStringLiteral("searchIndex")) ? Type::Dash : Type::ZDash;
 
+    createIndex();
+
     if (!dir.cd(QStringLiteral("Documents")))
         return;
 
-    m_keyword = (info.bundleName.isEmpty() ? m_name : info.bundleName).toLower();
+    //
+    // Setyp keywords
+    if (plist.contains(InfoPlist::DocSetPlatformFamily))
+        m_keywords << plist[InfoPlist::DocSetPlatformFamily].toString();
+
+    if (plist.contains(InfoPlist::DashDocSetPluginKeyword))
+        m_keywords << plist[InfoPlist::DashDocSetPluginKeyword].toString();
+
+    if (plist.contains(InfoPlist::DashDocSetKeyword))
+        m_keywords << plist[InfoPlist::DashDocSetKeyword].toString();
+
+    if (plist.contains(InfoPlist::DashDocSetFamily)) {
+        const QString kw = plist[InfoPlist::DashDocSetFamily].toString();
+        if (kw != QStringLiteral("dashtoc"))
+            m_keywords << kw;
+    }
+
+    /// TODO: Use 'unknown' instead of CFBundleName? (See #383)
+    m_keywords << plist.value(InfoPlist::CFBundleName, m_name).toString().toLower();
 
     // Try to find index path if metadata is missing one
     if (m_indexFilePath.isEmpty()) {
-        if (!info.indexFilePath.isEmpty() && dir.exists(info.indexFilePath))
-            m_indexFilePath = info.indexFilePath;
+        if (plist.contains(InfoPlist::DashIndexFilePath))
+            m_indexFilePath = plist[InfoPlist::DashIndexFilePath].toString();
         else if (dir.exists(QStringLiteral("index.html")))
             m_indexFilePath = QStringLiteral("index.html");
         else
@@ -113,9 +181,9 @@ QString Docset::title() const
     return m_title;
 }
 
-QString Docset::keyword() const
+QStringList Docset::keywords() const
 {
-    return m_keyword;
+    return m_keywords;
 }
 
 QString Docset::version() const
@@ -172,7 +240,7 @@ QList<SearchResult> Docset::search(const QString &query) const
     const SearchQuery searchQuery = SearchQuery::fromString(query);
     const QString sanitizedQuery = searchQuery.sanitizedQuery();
 
-    if (searchQuery.hasKeywords() && !searchQuery.hasKeyword(m_keyword))
+    if (searchQuery.hasKeywords() && !searchQuery.hasKeywords(m_keywords))
         return results;
 
     QString queryStr;
@@ -197,32 +265,38 @@ QList<SearchResult> Docset::search(const QString &query) const
                 notQuery = QString(" AND NOT (ztokenname LIKE '%1%' ESCAPE '\\' %2) ").arg(sanitizedQuery, subNames.arg("ztokenname", sanitizedQuery));
         }
         if (m_type == Docset::Type::Dash) {
-            queryStr = QString("SELECT name, path "
+            queryStr = QString("SELECT name, type, path "
                                "    FROM searchIndex "
                                "WHERE (name LIKE '%1%' ESCAPE '\\' %3) %2 "
-                               "LIMIT 100")
+                               "ORDER BY name COLLATE NOCASE LIMIT 100")
                     .arg(curQuery, notQuery, subNames.arg("name", curQuery));
         } else {
-            queryStr = QString("SELECT ztokenname, zpath, zanchor "
+            queryStr = QString("SELECT ztokenname, ztypename, zpath, zanchor "
                                "    FROM ztoken "
                                "JOIN ztokenmetainformation "
                                "    ON ztoken.zmetainformation = ztokenmetainformation.z_pk "
                                "JOIN zfilepath "
                                "    ON ztokenmetainformation.zfile = zfilepath.z_pk "
+                               "JOIN ztokentype "
+                               "    ON ztoken.ztokentype = ztokentype.z_pk "
                                "WHERE (ztokenname LIKE '%1%' ESCAPE '\\' %3) %2 "
-                               "LIMIT 100").arg(curQuery, notQuery,
-                                                subNames.arg("ztokenname", curQuery));
+                               "ORDER BY ztokenname COLLATE NOCASE LIMIT 100")
+                    .arg(curQuery, notQuery, subNames.arg("ztokenname", curQuery));
         }
 
         QSqlQuery query(queryStr, database());
         while (query.next()) {
             const QString itemName = query.value(0).toString();
-            QString path = query.value(1).toString();
-            if (m_type == Docset::Type::ZDash)
-                path += QLatin1Char('#') + query.value(1).toString();
+            QString path = query.value(2).toString();
+            if (m_type == Docset::Type::ZDash) {
+                const QString anchor = query.value(3).toString();
+                if (!anchor.isEmpty())
+                    path += QLatin1Char('#') + anchor;
+            }
 
             /// TODO: Third should be type
-            results.append(SearchResult{itemName, QString(), QString(),
+            results.append(SearchResult{itemName, QString(),
+                                        parseSymbolType(query.value(1).toString()),
                                         const_cast<Docset *>(this), path, sanitizedQuery});
         }
 
@@ -259,27 +333,27 @@ QList<SearchResult> Docset::relatedLinks(const QUrl &url) const
                                   "JOIN ztokenmetainformation ON ztoken.zmetainformation = ztokenmetainformation.z_pk "
                                   "JOIN zfilepath ON ztokenmetainformation.zfile = zfilepath.z_pk "
                                   "JOIN ztokentype ON ztoken.ztokentype = ztokentype.z_pk "
-                                  "WHERE zfilepath.zpath = \"%1\"");
+                                  "WHERE zfilepath.zpath = \"%1\" AND ztokenmetainformation.zanchor IS NOT NULL");
     }
 
     QSqlQuery query(queryStr.arg(cleanUrl.toString()), database());
 
     while (query.next()) {
-        QString sectionName = query.value(0).toString();
+        const QString sectionName = query.value(0).toString();
         QString sectionPath = query.value(2).toString();
         if (m_type == Docset::Type::ZDash) {
             sectionPath += QLatin1Char('#');
             sectionPath += query.value(3).toString();
         }
 
-        //QString parentName;
-        //normalizeName(sectionName, parentName);
-
         results.append(SearchResult{sectionName, QString(),
                                     parseSymbolType(query.value(1).toString()),
                                     const_cast<Docset *>(this), sectionPath, QString()});
     }
 
+    if (results.size() == 1)
+        results.clear();
+
     return results;
 }
 
@@ -288,22 +362,6 @@ QSqlDatabase Docset::database() const
     return QSqlDatabase::database(m_name, true);
 }
 
-void Docset::normalizeName(QString &name, QString &parentName)
-{
-    QRegExp matchMethodName(QStringLiteral("^([^\\(]+)(?:\\(.*\\))?$"));
-    if (matchMethodName.indexIn(name) != -1)
-        name = matchMethodName.cap(1);
-
-    const QStringList separators = {QStringLiteral("."), QStringLiteral("::"), QStringLiteral("/")};
-    for (const QString &sep : separators) {
-        if (name.indexOf(sep) != -1 && name.indexOf(sep) != 0) {
-            const QStringList splitted = name.split(sep);
-            name = splitted.at(splitted.size()-1);
-            parentName = splitted.at(splitted.size()-2);
-        }
-    }
-}
-
 void Docset::loadMetadata()
 {
     const QDir dir(m_path);
@@ -395,6 +453,40 @@ void Docset::loadSymbols(const QString &symbolType, const QString &symbolString)
         symbols.insertMulti(query.value(0).toString(), QDir(documentPath()).absoluteFilePath(query.value(1).toString()));
 }
 
+void Docset::createIndex()
+{
+    static const QString indexListQuery = QStringLiteral("PRAGMA INDEX_LIST('%1')");
+    static const QString indexDropQuery = QStringLiteral("DROP INDEX '%1'");
+    static const QString indexCreateQuery = QStringLiteral("CREATE INDEX IF NOT EXISTS %1%2"
+                                                           " ON %3 (name COLLATE NOCASE)");
+
+    QSqlQuery query(database());
+
+    const QString tableName = m_type == Type::Dash ? QStringLiteral("searchIndex")
+                                                   : QStringLiteral("ztoken");
+
+    query.exec(indexListQuery.arg(tableName));
+
+    QStringList oldIndexes;
+
+    while (query.next()) {
+        const QString indexName = query.value(1).toString();
+        if (!indexName.startsWith(IndexNamePrefix))
+            continue;
+
+        if (indexName.endsWith(IndexNameVersion))
+            return;
+
+        oldIndexes << indexName;
+    }
+
+    // Drop old indexes
+    for (const QString oldIndexName : oldIndexes)
+        query.exec(indexDropQuery.arg(oldIndexName));
+
+    query.exec(indexCreateQuery.arg(IndexNamePrefix, IndexNameVersion, tableName));
+}
+
 QString Docset::parseSymbolType(const QString &str)
 {
     /// Dash symbol aliases
